BEGIN;

-- Create a function that always returns the first non-NULL item
CREATE OR REPLACE FUNCTION public.first_agg ( anyelement, anyelement )
RETURNS anyelement LANGUAGE sql IMMUTABLE STRICT AS $$
        SELECT $1;
$$;
 
-- And then wrap an aggregate around it
CREATE AGGREGATE public.first (
        sfunc           = public.first_agg,
        basetype        = anyelement,
        stype           = anyelement
);
 
-- Create a function that always returns the last non-NULL item
CREATE OR REPLACE FUNCTION public.last_agg ( anyelement, anyelement )
RETURNS anyelement LANGUAGE sql IMMUTABLE STRICT AS $$
        SELECT $2;
$$;
 
-- And then wrap an aggregate around it
CREATE AGGREGATE public.last (
        sfunc           = public.last_agg,
        basetype        = anyelement,
        stype           = anyelement
);

CREATE TABLE component_version (
    component                   VARCHAR(32) NOT NULL,
    version                     VARCHAR(32) NOT NULL,
    modtime                     TIMESTAMP DEFAULT current_timestamp,
    CONSTRAINT component_version_uniq UNIQUE(component,version)
);

INSERT INTO component_version (component, version) VALUES ('ekmdata', '1.02a');

CREATE SEQUENCE ekmmeters_seq START 1;

CREATE TABLE ekmmeters (
    id                          SMALLINT NOT NULL DEFAULT nextval('ekmmeters_seq'),
    meteraddress                CHAR(12) NOT NULL,
    model                       INTEGER NOT NULL,
    modelname                   VARCHAR(64) NOT NULL,
    host                        VARCHAR(255) NOT NULL,
    port                        INTEGER NOT NULL DEFAULT 50000,
    password                    VARCHAR(8) DEFAULT NULL, -- null/default - no password set
    timezone                    VARCHAR(32) DEFAULT NULL, -- null/default - the same timezone as running host system
    description                 VARCHAR(255) NOT NULL,
    enabled                     BOOLEAN NOT NULL DEFAULT FALSE, -- disabled by default
    last_reset_resettable       TIMESTAMP WITH TIME ZONE DEFAULT NULL,
    last_reset_maxdemand        TIMESTAMP WITH TIME ZONE DEFAULT NULL,
    last_sync                   TIMESTAMP WITH TIME ZONE DEFAULT NULL,
    CONSTRAINT ekmmeters_meteraddress_uniq UNIQUE(meteraddress),
    CONSTRAINT ekmmeters_meteraddress_host_uniq UNIQUE(meteraddress,host)
);

-- My Home
-- INSERT INTO ekmmeters (meteraddress, model, host, port, password, timezone, description, modelname, enabled) VALUES ('000300000123', 1024, 'ekm.localhost.int', 50000, '12345678', 'America/New_York', 'Main Home EKM Meter, 20KVA main line feeds, 120/240v 3-wire', 'EKM Omnimeter Pulse V.4', true);

CREATE SEQUENCE ekmdata_seq START 1;

CREATE TABLE ekmdata (
    id                          INTEGER NOT NULL DEFAULT nextval('ekmdata_seq'),
    meterdate                   TIMESTAMP WITH TIME ZONE,
    computerdate                TIMESTAMP WITH TIME ZONE,
    model                       INTEGER,
    firmware                    INTEGER,
    meteraddress                VARCHAR(16) REFERENCES ekmmeters(meteraddress),
    totalkwh                    NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    forwardkwh                  NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    reversekwh                  NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    netkwh                      NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    totalkwht1                  NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    totalkwht2                  NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    totalkwht3                  NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    totalkwht4                  NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    reversekwht1                NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    reversekwht2                NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    reversekwht3                NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    reversekwht4                NUMERIC(12,2), --  up to 9,999,999,999.99 (kWh)
    voltage1                    NUMERIC(5,2), -- up to 999.99 (Volt)
    voltage2                    NUMERIC(5,2), -- up to 999.99 (Volt)
    voltage3                    NUMERIC(5,2), -- up to 999.99 (Volt)
    current1                    NUMERIC(6,2), -- up to 9,999.99 (Amp)
    current2                    NUMERIC(6,2), -- up to 9,999.99 (Amp)
    current3                    NUMERIC(6,2), -- up to 9,999.99 (Amp)
    power1                      NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    power2                      NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    power3                      NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    totalpower                  NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    cos1                        VARCHAR(6),
    cos1n                       NUMERIC(3,2),
    cos2                        VARCHAR(6),
    cos2n                       NUMERIC(3,2),
    cos3                        VARCHAR(6),
    cos3n                       NUMERIC(3,2),
    maxdemand                   NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    maxdemandperiod             INTEGER,
    ctratio                     INTEGER,
    pulse1count                 INTEGER,
    pulse1ratio                 INTEGER,
    pulse2count                 INTEGER,
    pulse2ratio                 INTEGER,
    pulse3count                 INTEGER,
    pulse3ratio                 INTEGER,
    reactivekhh                 NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    totalkwhl1                  NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    totalkwhl2                  NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    totalkwhl3                  NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    reversekwhl1                NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    reversekwhl2                NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    reversekwhl3                NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    resettabletotalkwh          NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    resettablereversekwh        NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    reactivepowerl1             NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    reactivepowerl2             NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    reactivepowerl3             NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    totalreactivepower          NUMERIC(9,2), -- up to 9,999,999.99 (Watt)
    frequency                   NUMERIC(5,2),-- up to 999.99 (Hz)
    pulseinput1                 VARCHAR(16),
    pulseinput2                 VARCHAR(16),
    pulseinput3                 VARCHAR(16),
    pulseinputstate1            VARCHAR(16),
    pulseinputstate2            VARCHAR(16),
    pulseinputstate3            VARCHAR(16),
    currentdirectionl1          VARCHAR(16),
    currentdirectionl2          VARCHAR(16),
    currentdirectionl3          VARCHAR(16),
    output1                     VARCHAR(16),
    output2                     VARCHAR(16),
    decimalplaces               INTEGER,
    autoresetmaxdemand          INTEGER,
    settablepulseoutputratio    INTEGER,
    valid                       INTEGER,
    error                       VARCHAR(32),
    CONSTRAINT ekmdata_id PRIMARY KEY(id)
);

CREATE INDEX ekmdata_meteraddress_idx on ekmdata(meteraddress);

CREATE INDEX ekmdata_meterdate_idx on ekmdata(meterdate);

COMMIT;
